@@ -25,12 +25,10 @@ class BetweennessEffect : public NetworkEffect
 public:
 	BetweennessEffect(const EffectInfo * pEffectInfo);
 
-	virtual double calculateTieFlipContribution(int alter) const;
-	virtual bool usesTable(const ConfigurationTable * pTable) const;
+	virtual double calculateContribution(int alter) const;
 
 protected:
-	virtual double statistic(Network * pNetwork,
-		Network * pSummationTieNetwork) const;
+	virtual double statistic(const Network * pSummationTieNetwork) const;
 };
 
 }
