@@ -10,7 +10,7 @@
  *****************************************************************************/
 
 #include "CovariateAlterEffect.h"
-#include "data/Network.h"
+#include "network/Network.h"
 #include "model/variables/NetworkVariable.h"
 
 namespace siena
@@ -32,7 +32,7 @@ CovariateAlterEffect::CovariateAlterEffect(const EffectInfo * pEffectInfo,
 /**
  * Calculates the contribution of a tie flip to the given actor.
  */
-double CovariateAlterEffect::calculateTieFlipContribution(int alter) const
+double CovariateAlterEffect::calculateContribution(int alter) const
 {
 	double change = this->value(alter);
 
@@ -41,23 +41,18 @@ double CovariateAlterEffect::calculateTieFlipContribution(int alter) const
 		change *= change;
 	}
 
-	if (this->pVariable()->outTieExists(alter))
-	{
-		// The ego would loose the tie and the covariate value of the alter
-		change = -change;
-	}
-
 	return change;
 }
 
 
 /**
- * Returns the statistic corresponding to this effect as part of
- * the evaluation function with respect to the given network.
+ * Detailed comment in the base class.
  */
-double CovariateAlterEffect::evaluationStatistic(Network * pNetwork) const
+double CovariateAlterEffect::statistic(const Network * pSummationTieNetwork)
+	const
 {
 	double statistic = 0;
+	const Network * pNetwork = this->pNetwork();
 
 	for (int i = 0; i < pNetwork->m(); i++)
 	{
@@ -70,28 +65,11 @@ double CovariateAlterEffect::evaluationStatistic(Network * pNetwork) const
 				value *= value;
 			}
 
-			statistic += pNetwork->inDegree(i) * value;
+			statistic += pSummationTieNetwork->inDegree(i) * value;
 		}
 	}
 
 	return statistic;
 }
 
-
-/**
- * Returns the statistic corresponding to this effect as part of
- * the endowment function with respect to an initial network
- * and a network of lost ties. The current network is implicit as
- * the introduced ties are not relevant for calculating
- * endowment statistics.
- */
-double CovariateAlterEffect::endowmentStatistic(Network * pInitialNetwork,
-	Network * pLostTieNetwork) const
-{
-	// This is the same as the evaluation statistic computed with respect
-	// to the network of lost ties.
-
-	return this->evaluationStatistic(pLostTieNetwork);
-}
-
 }
