@@ -10,9 +10,9 @@
  *****************************************************************************/
 
 #include "CovariateEgoAlterEffect.h"
-#include "data/Network.h"
-#include "data/IncidentTieIterator.h"
-#include "data/CommonNeighborIterator.h"
+#include "network/Network.h"
+#include "network/IncidentTieIterator.h"
+#include "network/CommonNeighborIterator.h"
 #include "model/variables/NetworkVariable.h"
 
 namespace siena
@@ -36,19 +36,13 @@ CovariateEgoAlterEffect::CovariateEgoAlterEffect(
 /**
  * Calculates the contribution of a tie flip to the given actor.
  */
-double CovariateEgoAlterEffect::calculateTieFlipContribution(int alter) const
+double CovariateEgoAlterEffect::calculateContribution(int alter) const
 {
 	double change = 0;
 
-	if (!this->lreciprocal || this->pVariable()->inTieExists(alter))
+	if (!this->lreciprocal || this->inTieExists(alter))
 	{
-		change = this->value(this->pVariable()->ego()) * this->value(alter);
-
-		if (this->pVariable()->outTieExists(alter))
-		{
-			// The ego would loose the tie, so the contribution is the opposite
-			change = -change;
-		}
+		change = this->value(this->ego()) * this->value(alter);
 	}
 
 	return change;
@@ -58,10 +52,11 @@ double CovariateEgoAlterEffect::calculateTieFlipContribution(int alter) const
 /**
  * Detailed comment in the base class.
  */
-double CovariateEgoAlterEffect::statistic(Network * pNetwork,
-	Network * pSummationTieNetwork) const
+double CovariateEgoAlterEffect::statistic(const Network * pSummationTieNetwork)
+	const
 {
 	double statistic = 0;
+	const Network * pNetwork = this->pNetwork();
 	int n = pNetwork->n();
 
 	for (int i = 0; i < n; i++)
