@@ -25,11 +25,9 @@ class CovariateIndirectTiesEffect : public CovariateDependentNetworkEffect
 public:
 	CovariateIndirectTiesEffect(const EffectInfo * pEffectInfo);
 
-	virtual double calculateTieFlipContribution(int alter) const;
-	virtual bool usesTable(const ConfigurationTable * pTable) const;
-	virtual double evaluationStatistic(Network * pNetwork) const;
-	virtual double endowmentStatistic(Network * pInitialNetwork,
-		Network * pLostTieNetwork) const;
+	virtual double calculateContribution(int alter) const;
+	virtual double evaluationStatistic() const;
+	virtual double endowmentStatistic(Network * pLostTieNetwork) const;
 };
 
 }
