@@ -26,12 +26,10 @@ class DenseTriadsEffect : public NetworkEffect
 public:
 	DenseTriadsEffect(const EffectInfo * pEffectInfo);
 
-	virtual double calculateTieFlipContribution(int alter) const;
-	virtual bool usesTable(const ConfigurationTable * pTable) const;
+	virtual double calculateContribution(int alter) const;
 
 protected:
-	virtual double statistic(Network * pNetwork,
-		Network * pSummationTieNetwork) const;
+	virtual double statistic(const Network * pSummationTieNetwork) const;
 
 private:
 	int ldensity;
