@@ -1,10 +1,10 @@
 /******************************************************************************
  * SIENA: Simulation Investigation for Empirical Network Analysis
- * 
+ *
  * Web: http://www.stats.ox.ac.uk/~snijders/siena/
- * 
+ *
  * File: DensityEffect.h
- * 
+ *
  * Description: This file contains the declaration of the class DensityEffect.
  *****************************************************************************/
 
@@ -23,11 +23,11 @@ class DensityEffect : public NetworkEffect
 {
 public:
 	DensityEffect(const EffectInfo * pEffectInfo);
-	
-	virtual double calculateTieFlipContribution(int alter) const;
-	virtual double evaluationStatistic(Network * pNetwork) const;
-	virtual double endowmentStatistic(Network * pInitialNetwork,
-		Network * pLostTieNetwork) const;
+
+	virtual double calculateContribution(int alter) const;
+
+protected:
+	virtual double statistic(const Network * pSummationTieNetwork) const;
 };
 
 }
