@@ -27,10 +27,9 @@ class InverseSquaredOutdegreeEffect : public NetworkEffect
 public:
 	InverseSquaredOutdegreeEffect(const EffectInfo * pEffectInfo);
 
-	virtual double calculateTieFlipContribution(int alter) const;
-	virtual double evaluationStatistic(Network * pNetwork) const;
-	virtual double endowmentStatistic(Network * pInitialNetwork,
-		Network * pLostTieNetwork) const;
+	virtual double calculateContribution(int alter) const;
+	virtual double evaluationStatistic() const;
+	virtual double endowmentStatistic(Network * pLostTieNetwork) const;
 
 private:
 	double lc;
