@@ -1,10 +1,10 @@
 /******************************************************************************
  * SIENA: Simulation Investigation for Empirical Network Analysis
- * 
+ *
  * Web: http://www.stats.ox.ac.uk/~snijders/siena/
- * 
+ *
  * File: OutdegreeActivityEffect.h
- * 
+ *
  * Description: This file contains the definition of the
  * OutdegreeActivityEffect class.
  *****************************************************************************/
@@ -26,11 +26,11 @@ class OutdegreeActivityEffect : public NetworkEffect
 {
 public:
 	OutdegreeActivityEffect(const EffectInfo * pEffectInfo);
-	
-	virtual double calculateTieFlipContribution(int alter) const;
-	virtual double evaluationStatistic(Network * pNetwork) const;
-	virtual double endowmentStatistic(Network * pInitialNetwork,
-		Network * pLostTieNetwork) const;
+
+	virtual double calculateContribution(int alter) const;
+
+protected:
+	virtual double statistic(const Network * pSummationTieNetwork) const;
 };
 
 }
