@@ -25,11 +25,10 @@ class SameCovariateEffect : public CovariateDependentNetworkEffect
 public:
 	SameCovariateEffect(const EffectInfo * pEffectInfo, bool reciprocal);
 
-	virtual double calculateTieFlipContribution(int alter) const;
+	virtual double calculateContribution(int alter) const;
 
 protected:
-	virtual double statistic(Network * pNetwork,
-		Network * pSummationTieNetwork) const;
+	virtual double statistic(const Network * pSummationTieNetwork) const;
 
 private:
 	// Indicates if the reciprocal version of the same covariate effect is
